package Alfabeto;

################################################################################
# Alfabeto.pm
#
# Auxilia na manipulao da lngua atravs de um alfabeto personalizvel
#
# Autor: Rafael Giusti [rg.bacs@gmail.com]
#
################################################################################



use strict;


################################################################################
# Mtodo:	new
# Propsito:	Constri um novo objeto da classe
# Entrada:	--
# Sada:	Um objeto da classe Alfabeto
# Detalhes:	--
#
################################################################################

sub new {
	my $proto = shift;
	my $class = ref $proto || $proto;

	my $self = bless {
		Conhecidos	=> {},  # Todos os que aparecen no arquivo de alfabeto
		Minimizar	=> {},  # Relao maiscula --> minscula
		Ordenar		=> {},  # Conjuntos de letras com valor de ordem
		Ignorar		=> {},  # Smbolos desconsiderados na ordenao
		Limpar		=> "",  # Smbolos a serem removidos de uma string
		Quebrar		=> "",	# Delimitadores de palavras
	}, $proto;

	return $self;
}



################################################################################
# Mtodo:	caracterizar
# Propsito:	Retorna um caractere, convertendo seqncias de escape
# Entrada:	Uma string que contm um caracter ou uma seqncia de escape
# Sada:	Um escalar que contm um nico caracter
# Detalhes:	--
#
################################################################################

sub caracterizar {
	my $self = shift;
	my $car = shift;

	if ($car =~ /\\u{([0-9a-fA-F]+)}/) {
		return sprintf "%c", hex $1;
	}
	else {
		return $car;
	}
}



################################################################################
# Mtodo:	proteger
# Propsito:	Assegura que um smbolo pode ser corretamente usado num conjunto
# 		[] de regexp
# Entrada:	Um smbolo
# Sada:	O smbolo ou uma seqncia de escape que represente o smbolo
# Detalhes:	--
#
################################################################################

my %trocas = (
	"["	=>	"\\[",
	"]"	=>	"\\]",
	"-"	=>	"\\-",
	"^"	=>	"\\^"
);

sub proteger {
	my $self    = shift;
	my $simbolo = shift;

	if (defined $trocas{$simbolo}) {
		return $trocas{$simbolo};
	}
	else {
		return $simbolo;
	}
}



################################################################################
# Mtodo:	carregar
# Propsito:	Carrega informaes de aflabeto a partir de um arquivo	
# Entrada:	Uma string que contm o caminho do arquivo a ser carregado
# Sada:	--	
# Detalhes:	--
#
################################################################################

sub carregar {
	my $self = shift;
	my $arquivo = shift;

	my $escopo = "nenhum";
	my $numlinha = 0;
	my $ordem = 0;

	my %escoposvalidos = (
		"MINIMIZAR" => 1, 
		"ORDEM" => 1,
		"IGNORAR" => 1,
		"LIMPAR" => 1,
		"QUEBRAR" => 1
	);
	
	# Limpa as configuraes para que o alfabeto possa ser trocado
	# dinamicamente
	#
	$self->{Minimizar} = {};
	$self->{Ordenar} = {};
	$self->{Ignorar} = {};
	$self->{Limpar} = "";
	$self->{Quebrar} = "";

	open ARQUIVO, "<:utf8", $arquivo or die "Cannot open $arquivo: $!";
	while (!eof ARQUIVO) {
		my $linha = <ARQUIVO>;
		$numlinha++;
		chomp $linha;
		$linha =~ s/#.*$//;
		$linha =~ s/^[\s\t]+//;
		next unless $linha =~ /[^\s\t]/;

		if ($linha =~ /::([A-Z]+)$/) {
			my $novoescopo = $1;
			if ($1 eq "FIM") {
				last;
			}
			elsif (defined $escoposvalidos{$novoescopo}) {
				$escopo = $novoescopo;
			}
			elsif ($novoescopo !~ /[^\t\s]/) {
				die "Escopo nulo na linha $numlinha do alfabeto $arquivo\n";
			}
			else { 
				die "Escopo desconhecido ($novoescopo) na linha $numlinha do alfabeto $arquivo\n";
			}
		}
		elsif ($escopo eq "MINIMIZAR") {
			$linha =~ /([^\s]+)\s+([^\s]+)/;
			$self->{Minimizar}->{$self->caracterizar($1)} = $self->caracterizar($2);
			$self->{Conhecidos}->{$self->caracterizar($1)} = 1;
			$self->{Conhecidos}->{$self->caracterizar($2)} = 1;
		}
		elsif ($escopo eq "ORDEM") {
			# O primeiro elemento do escopo ordenar tem ordem zero. Os elementos
			# seguintes tm ordem incrementada. Quanto menor a ordem, maior
			# a precedncia do caracter
			#
			foreach my $letra (split /[\s\t]/, $linha) {
				$self->{Ordenar}->{$self->caracterizar($letra)} = $ordem;
				$self->{Conhecidos}->{$self->caracterizar($letra)} = 1;
			}
			$ordem++;
		}
		elsif ($escopo eq "IGNORAR") {
			$self->{Ignorar}->{$self->caracterizar($linha)} = 1;
			$self->{Conhecidos}->{$self->caracterizar($linha)} = 1;
		}
		elsif ($escopo eq "LIMPAR") {
			$self->{Limpar} = $self->{Limpar} . $self->proteger($self->caracterizar($linha));
			$self->{Conhecidos}->{$self->caracterizar($linha)} = 1;
		}
		elsif ($escopo eq "QUEBRAR") {
			$self->{Quebrar} = $self->{Quebrar} . $self->proteger($self->caracterizar($linha));
			$self->{Conhecidos}->{$self->caracterizar($linha)} = 1;
		}
		else {
			die "Escopo indefinido na linha $numlinha do alfabeto $arquivo\n";
		}
	}	
	close ARQUIVO;
}



################################################################################
# Mtodo:	procurarnovos
# Propsito:	Procura smbolos desconhecidos numa string
# Entrada:	Uma string
# Sada:	Uma lista contendo todos os smbolos da string que no esto
# 		registados no alfabeto	
# Detalhes:	--
#
################################################################################

sub procurarnovos {
	my $self = shift;
	my $linha = shift;

	$linha =~ s/[\s\t\r\n]//g;
	my @linha = split //, $linha;

	my @novos;
	foreach my $letra (@linha) {
		push @novos, $letra unless defined $self->{Conhecidos}->{$letra} ;
	}

	return @novos;
}



################################################################################
# Mtodo:	_avancar [VISIBILIDADE PRIVADA (PRIVATE)]
# Propsito:	Utilizado durante a comparao de duas strings: busca o prximo
# 		caracter relevante para a tarefa de comparao, ignorando 
# 		smbolos desconheciso ou listados no campo ::IGNORAR
# Entrada:	1. Uma referncia para o ndice do caracter
# 		2. Uma referncia para a lista de caracteres
# Sada:	--
# Detalhes:	Verifique os detalhes do mtodo comparar
#
################################################################################

sub _avancar {
	my $self = shift;
	my $indice = shift;
	my $lista = shift;

	do {
		$$indice++;
	} while (($$indice < scalar @$lista) && ($self->{Ignorar}->{$lista->[$$indice]} || 
			!defined $self->{Ordenar}->{$lista->[$$indice]}));
}



################################################################################
# Mtodo:	comparar
# Propsito:	Compara duas strings segundo as regras especificadas no arquivo
# 		do alfabeto
# Entrada:	1. Uma string A
# 		2. Uma string B
# Sada:	-1 se A  anterior a B na ordem lexicogrfica
# 		 0 se A e B tm mesma ordem (NO implica A e B serem idnticas)
# 		+1 se A  posterior a B na ordem lexicogrfica
# Detalhes:	As duas strings so quebradas em caracteres, os quais so
# 		armazenados em listas. O mtodo _avancar  utilizado para iterar
# 		a lista. Somente caracteres relevantes para a comparao so
# 		verificados
#
################################################################################

sub comparar {
	my $self = shift;
	my @a = split //, shift;
	my @b = split //, shift;
	my $i = -1;
	my $j = -1;

	# Avana os ndices at encontrar dois caracteres que possuem ordens 
	# diferentes ou at que uma das listas seja inteiramente consumida
	#
	$self->_avancar(\$i, \@a);
	$self->_avancar(\$j, \@b);
	while (($i < scalar @a) && ($j < scalar @b) && 
			$self->{Ordenar}->{$a[$i]} == $self->{Ordenar}->{$b[$j]}) {
		$self->_avancar(\$i, \@a);
		$self->_avancar(\$j, \@b);
	}

	if ($i == scalar @a) {
		if ($j == scalar @b) {
			# Se ambas as listas foram inteiramente consumidas, as 
			# duas strings possuem mesma ordem
			#
			return 0;
		}
		else {
			# Se a string A foi inteiramente consumida, mas a string
			# B no, ento A  substring de B e vem antes na ordem
			# lexicogrfica (ex: bola e bolacha)
			#
			return -1;
		}
	}
	else {
		if ($j == scalar @b) {
			# Se a string B foi inteiramente consumida, mas a string
			# A no, ento B  substring de A e vem antes na ordem
			# lexicogrfica (ex: bolacha e bola)
			#
			return 1;
		}
		else {
			#return (ord $a[$i]) - (ord $b[$j]);
			return $self->{Ordenar}->{$a[$i]} - $self->{Ordenar}->{$b[$j]};
		}
	}
}



################################################################################
# Mtodo:	minimizar
# Propsito:	Minimiza todos os caracteres de uma string utilizando as
# 		regras do alfabeto
# Entrada:	Uma string
# Sada:	A mesma string com todos os caracteres minimizados	
# Detalhes:	--
#
################################################################################

sub minimizar {
	my $self = shift;
	my @frase = split //, shift;
	
	my $saida = "";
	foreach my $letra (@frase) {
		if (defined $self->{Minimizar}->{$letra}) {
			$saida = $saida . $self->{Minimizar}->{$letra};
		}
		else {
			$saida = $saida . $letra;
		}
	}

	return $saida;
}



################################################################################
# Mtodo:	limpar
# Propsito:	Remove da strings caracteres listados no campo ::LIMPAR	
# Entrada:	Uma string
# Sada:	Uma string, com vrios smbolos removidos	
# Detalhes:	--
#
################################################################################

sub limpar {
	my $self = shift;
	my $str  = shift;
	my ($limpar);

	$limpar = "[$self->{Limpar}]";
	$str =~ s/$limpar//g;
	return $str;
}



################################################################################
# Mtodo:	quebrar
# Propsito:	Quebra uma string em vrias substrings em pontos definidos
# 		no campo ::QUEBRAR
# Entrada:	Uma string
# Sada:	Uma lista de palavras da string, sem brancos
# Detalhes:	--
#
################################################################################

sub quebrar {
	my $self = shift;
	my $str  = shift;
	my ($quebrar, @lista);

	$quebrar = "[$self->{Quebrar}]";
	@lista = split /$quebrar/, $str;
	return @lista;
}



return 1;
